/*
 * The Modern Classic Game - a 3D/2D game
 * Copyright (C) 2013 Thiago Lasso (SoniEx2)
 * 
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */
package com.github.soniex2.tmcg.entity;

/**
 * @author SoniEx2
 */
public abstract class Entity {

	/**
	 * This entity's X axis movement. Implementation note: This should be
	 * calculated relative to this entity's X rotation.
	 */
	protected double movX;
	/**
	 * This entity's Y axis movement. Implementation note: This should be
	 * calculated relative to this entity's Y rotation.
	 */
	protected double movY;
	/**
	 * This entity's Z axis movement. Implementation note: This should be
	 * calculated relative to this entity's Z rotation.
	 */
	protected double movZ;
	/**
	 * This entity's X axis angle.
	 */
	protected double rotX;
	/**
	 * This entity's Y axis angle.
	 */
	protected double rotY;
	/**
	 * This entity's Z axis angle.
	 */
	protected double rotZ;
	/**
	 * This entity's X position.
	 */
	protected double x;
	/**
	 * This entity's Y position.
	 */
	protected double y;
	/**
	 * This entity's Z position.
	 */
	protected double z;

	public double getMovX() {
		return movX;
	}

	public double getMovY() {
		return movY;
	}

	public double getMovZ() {
		return movZ;
	}

	/**
	 * Gets this entity's X axis angle.
	 * 
	 * @return this entity's X axis angle.
	 */
	public double getRotX() {
		return rotX;
	}

	/**
	 * Gets this entity's Y axis angle.
	 * 
	 * @return this entity's Y axis angle.
	 */
	public double getRotY() {
		return rotY;
	}

	/**
	 * Gets this entity's Z axis angle.
	 * 
	 * @return this entity's Z axis angle.
	 */
	public double getRotZ() {
		return rotZ;
	}

	/**
	 * Gets this entity's X position.
	 * 
	 * @return this entity's X position.
	 */
	public double getX() {
		return x;
	}

	/**
	 * Gets this entity's Y position.
	 * 
	 * @return this entity's Y position.
	 */
	public double getY() {
		return y;
	}

	/**
	 * Gets this entity's Z position.
	 * 
	 * @return this entity's Z position.
	 */
	public double getZ() {
		return z;
	}

	public void setMovX(double movX) {
		this.movX = movX;
	}

	public void setMovY(double movY) {
		this.movY = movY;
	}

	public void setMovZ(double movZ) {
		this.movZ = movZ;
	}

	/**
	 * Sets this entity's position.
	 * 
	 * @param x
	 *            - the X position.
	 * @param y
	 *            - the Y position.
	 * @param z
	 *            - the Z position.
	 */
	public void setPos(double x, double y, double z) {
		this.x = x;
		this.y = y;
		this.z = z;
	}

	/**
	 * Sets this entity's angles.
	 * 
	 * @param rotX
	 *            - the X axis angle.
	 * @param rotY
	 *            - the Y axis angle.
	 * @param rotZ
	 *            - the Z axis angle.
	 */
	public void setRotation(double rotX, double rotY, double rotZ) {
		this.rotX = rotX;
		this.rotY = rotY;
		this.rotZ = rotZ;
	}

	/**
	 * Sets this entity's angles.
	 * 
	 * @param movX
	 *            - the X axis movement.
	 * @param movY
	 *            - the Y axis movement.
	 * @param movZ
	 *            - the Z axis movement.
	 */
	public void setMovement(double movX, double movY, double movZ) {
		this.movX = movX;
		this.movY = movY;
		this.movZ = movZ;
	}

	/**
	 * Sets this entity's X axis angle.
	 * 
	 * @param rotX
	 *            - the X axis angle.
	 */
	public void setRotX(double rotX) {
		this.rotX = rotX;
	}

	/**
	 * Sets this entity's Y axis angle.
	 * 
	 * @param rotY
	 *            - the Y axis angle.
	 */
	public void setRotY(double rotY) {
		this.rotY = rotY;
	}

	/**
	 * Sets this entity's Z axis angle.
	 * 
	 * @param rotZ
	 *            - the Z axis angle.
	 */
	public void setRotZ(double rotZ) {
		this.rotZ = rotZ;
	}

	/**
	 * Sets this entity's X position.
	 * 
	 * @param x
	 *            - the X position.
	 */
	public void setX(double x) {
		this.x = x;
	}

	/**
	 * Sets this entity's Y position.
	 * 
	 * @param x
	 *            - the Y position.
	 */
	public void setY(double y) {
		this.y = y;
	}

	/**
	 * Sets this entity's Z position.
	 * 
	 * @param x
	 *            - the Z position.
	 */
	public void setZ(double z) {
		this.z = z;
	}

	/**
	 * Updates this entity. This should be called 20 times per second.
	 */
	public abstract void update();

	/**
	 * Renders this entity.
	 * 
	 * @param in2D
	 *            - if this entity should be rendered in 2D or 3D.
	 */
	public abstract void render(boolean in2D);
}
