package com.github.soniex2.tmcg.ym2612;

import java.io.Closeable;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioFormat.Encoding;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.SourceDataLine;

public class JavaxOutputDac extends OutputDac implements Closeable {

	private SourceDataLine[] out = new SourceDataLine[6];

	public JavaxOutputDac(int rate) {
		AudioFormat af = new AudioFormat(Encoding.PCM_SIGNED, rate, 16, 2, 4,
				rate, true);
		for (int x = 0; x < 6; x++) {
			try {
				out[x] = AudioSystem.getSourceDataLine(af);
				out[x].open();
			} catch (LineUnavailableException e) {
				e.printStackTrace();
			}
		}
	}

	private byte[] temp = new byte[2];
	private byte[][] temp2 = new byte[6][4];

	@Override
	public void set(int i, int channel, boolean left, boolean right) {
		i &= (1 << 9) - 1;
		if (!left && !right) {
			temp2[channel][0] = temp2[channel][2] = 0;
			temp2[channel][1] = temp2[channel][3] = 0;
			return;
		}
		short s = (short) (i << 7);
		temp[0] = (byte) ((s & 0xFF00) >> 8);
		temp[1] = (byte) (s & 0xFF);
		if (left && right) {
			temp2[channel][0] = temp2[channel][2] = temp[0];
			temp2[channel][1] = temp2[channel][3] = temp[1];
		} else if (left) {
			temp2[channel][0] = temp[0];
			temp2[channel][1] = temp[1];
			temp2[channel][2] = 0;
			temp2[channel][3] = 0;
		} else if (right) {
			temp2[channel][0] = 0;
			temp2[channel][1] = 0;
			temp2[channel][2] = temp[0];
			temp2[channel][3] = temp[1];
		}
	}

	public void play() {
		for (int x = 0; x < 6; x++) {
			out[x].start();
		}
	}

	public void stop() {
		for (int x = 0; x < 6; x++) {
			out[x].stop();
		}
	}

	@Override
	public void close() {
		for (int x = 0; x < 6; x++) {
			out[x].close();
			out[x] = null;
		}
	}

	@Override
	public void update(int channel) {
		out[channel].write(temp2[channel], 0, 4);
	}

	public void drain() {
		for (int x = 0; x < 6; x++) {
			out[x].drain();
		}
	}

}
