package com.github.soniex2.tmcg.ym2612;

public class PhaseGenerator {

	private static int[][] detuneKeycode = new int[][] { // Detune Keycode Shift
															// Thingy
			{ 0, 0, 1, 2 }, // 0 (0x00)
			{ 0, 0, 1, 2 }, // 1 (0x01)
			{ 0, 0, 1, 2 }, // 2 (0x02)
			{ 0, 0, 1, 2 }, // 3 (0x03)
			{ 0, 1, 2, 2 }, // 4 (0x04)
			{ 0, 1, 2, 3 }, // 5 (0x05)
			{ 0, 1, 2, 3 }, // 6 (0x06)
			{ 0, 1, 2, 3 }, // 7 (0x07)
			{ 0, 1, 2, 4 }, // 8 (0x08)
			{ 0, 1, 3, 4 }, // 9 (0x09)
			{ 0, 1, 3, 4 }, // 10 (0x0A)
			{ 0, 1, 3, 5 }, // 11 (0x0B)
			{ 0, 2, 4, 5 }, // 12 (0x0C)
			{ 0, 2, 4, 6 }, // 13 (0x0D)
			{ 0, 2, 4, 6 }, // 14 (0x0E)
			{ 0, 2, 5, 7 }, // 15 (0x0F)
			{ 0, 2, 5, 8 }, // 16 (0x10)
			{ 0, 3, 6, 8 }, // 17 (0x11)
			{ 0, 3, 6, 9 }, // 18 (0x12)
			{ 0, 3, 7, 10 }, // 19 (0x13)
			{ 0, 4, 8, 11 }, // 20 (0x14)
			{ 0, 4, 8, 12 }, // 21 (0x15)
			{ 0, 4, 9, 13 }, // 22 (0x16)
			{ 0, 5, 10, 14 }, // 23 (0x17)
			{ 0, 5, 11, 16 }, // 24 (0x18)
			{ 0, 6, 12, 17 }, // 25 (0x19)
			{ 0, 6, 13, 19 }, // 26 (0x1A)
			{ 0, 7, 14, 20 }, // 27 (0x1B)
			{ 0, 8, 16, 22 }, // 28 (0x1C)
			{ 0, 8, 16, 22 }, // 29 (0x1D)
			{ 0, 8, 16, 22 }, // 30 (0x1E)
			{ 0, 8, 16, 22 } // 31 (0x1F)
	};

	private int block;
	private int detune;
	private int fnum = 0;
	private int multiplier;

	public void setDetune(int n) {
		detune = n & 0b111;
	}

	public void setMultiplier(int m) {
		multiplier = m & 0xF;
	}

	public void setBlock(int b) {
		block = b & 0b111;
	}
	
	public void setFnum(int fnum) {
		this.fnum = fnum;
	}

	public int get() {
		return fnum;
	}

	protected boolean fnum(int bit) {
		return (fnum & (1 << bit)) != 0;
	}

	public void update() {
		int nfnum = (fnum << block) >> 1;
		int kcode = (block << 2)
				| (fnum(10) ? 1 : 0) << 1
				| ((fnum(10) && (fnum(9) || fnum(8) || fnum(7)) || !fnum(10)
						&& fnum(9) && fnum(8) && fnum(7)) ? 1 : 0);
		boolean detb = (detune & 0b100) != 0;
		int detune = this.detune & 0b11;
		int shift = detuneKeycode[kcode][detune];
		if (detb) {
			shift = 0 - shift;
			shift &= (1 << 17) - 1;
		}
		nfnum = nfnum + shift;
		nfnum &= (1 << 17) - 1;
		if (multiplier == 0) {
			fnum = nfnum >> 1;
		} else {
			fnum = nfnum * multiplier;
		}
		fnum &= (1 << 20) - 1;
	}

}
